package com.mobile.service;

import java.util.Map;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import javax.servlet.http.HttpServletRequest;
import org.apache.log4j.Logger;

import com.mobile.message.request.*;
import com.mobile.message.response.Article;
import com.mobile.message.response.NewsResponse;
import com.mobile.message.response.TextResponse;
import com.mobile.util.HttpTools;
import com.mobile.util.MessageUtil;

/**
 * 核心服务类
 *
 * @author liufeng,eXcel
 * @date 2013-05-20,
 * @update 2016-10-1
 *
 */
public class CoreService {
    /**
     * 处理微信发来的请求
     *
     * @param request
     * @return
     */

    //定义一个logger,用于发送日志,可以在console或者服务器上看到。存储目录在log4j.properties中定义
    private static final Logger logger = Logger.getLogger(CoreService.class);

    //处理请求
    public static String processRequest(HttpServletRequest request){
        String returnMsg=null;
        try{
            Map<String, String> requestMap = MessageUtil.parseXml(request); //解析XML
            String msgType = requestMap.get("MsgType"); //消息类型
            logger.info("***step1 msgType=" + msgType);
            switch (msgType){
                case MessageUtil.REQ_MESSAGE_TYPE_TEXT:
                    returnMsg=HandleTextMsg(new TextMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_IMAGE:
                    returnMsg=HandleImageMsg(new ImageMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_LINK:
                    returnMsg=HandleLinkMsg(new LinkMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_LOCATION:
                    returnMsg=HandleLocationMsg(new LocationMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_SHORT_VIDEO:
                    returnMsg=HandleShortVideoMsg(new ShortVideoMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_VIDEO:
                    returnMsg=HandleVideoMsg(new VideoMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_VOICE:
                    returnMsg=HandleVoiceMsg(new VoiceMessage(requestMap));
                    break;
                case MessageUtil.REQ_MESSAGE_TYPE_EVENT:
                    // 事件类型
                    String eventType = requestMap.get("Event");
                    switch (eventType){
                        case MessageUtil.EVENT_TYPE_SUBSCRIBE:
                            returnMsg=HandleSubscribeEvent(new BaseMessage(requestMap));
                            break;
                        case MessageUtil.EVENT_TYPE_UNSUBSCRIBE:
                            returnMsg=HandleUnsubscribeEvent(new BaseMessage(requestMap));
                            break;
                        case MessageUtil.EVENT_TYPE_CLICK:
                            returnMsg=HandleClickEvent(new BaseMessage(requestMap));
                            break;
                        default:
                            returnMsg=HandleDefaultMsgOrEvent(new BaseMessage(requestMap));
                    }
                    break;
                default:
                    returnMsg=HandleDefaultMsgOrEvent(new BaseMessage(requestMap));
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
        return returnMsg;
    }

    public static String HandleTextMsg(TextMessage textMessage)
    {
        String respContent = "您发送的是文本消息!";
        if (textMessage.getContent().indexOf("/:") > -1)
            respContent="你"+textMessage.getContent()+"什么?";
        TextResponse textResponse=new TextResponse(textMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleImageMsg(ImageMessage imageMessage)
    {
        String title="鉴定结果";
        String picUrl=imageMessage.getPicUrl();
        String forwardUrl= HttpTools.imagePlusPlusPath+picUrl;
        logger.info("***step2 image++ Url=" + forwardUrl);

        //从image++网站识别上传的图片的拍摄地点、拍摄物体。
        String imagePlusPlusReturn=getHttpResponse(forwardUrl);
        logger.info("***step3 image++ Response=" + imagePlusPlusReturn);
        String description = ReadJSONString(imagePlusPlusReturn);
        logger.info("***step4 image++ Response=" + description);

        Article article=new Article(title,description,picUrl,forwardUrl);
        NewsResponse newsResponse=new NewsResponse(imageMessage);
        newsResponse.addArticle(article);
        String respMsg=MessageUtil.responseToXml(newsResponse);
        logger.info("***step5  Weixin Response=" + respMsg);
        return respMsg;
    }
    public static String HandleLinkMsg(LinkMessage linkMessage)
    {
        String respContent = "您发送的是链接!";
        TextResponse textResponse=new TextResponse(linkMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleLocationMsg(LocationMessage locationMessage)
    {
        String respContent = "您发送的是地理位置!";
        TextResponse textResponse=new TextResponse(locationMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleShortVideoMsg(ShortVideoMessage shortVideoMessage)
    {
        String respContent = "您发送的是小视频!";
        TextResponse textResponse=new TextResponse(shortVideoMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleVideoMsg(VideoMessage videoMessage)
    {
        String respContent = "您发送的是视频!";
        TextResponse textResponse=new TextResponse(videoMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleVoiceMsg(VoiceMessage voiceMessage)
    {
        String respContent = "您发送的是语音!";
        TextResponse textResponse=new TextResponse(voiceMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleSubscribeEvent(BaseMessage baseMessage)
    {
        String respContent = "欢迎订阅！";
        TextResponse textResponse=new TextResponse(baseMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleUnsubscribeEvent(BaseMessage baseMessage)
    {
        String respContent = "再见,希望您再来!";
        TextResponse textResponse=new TextResponse(baseMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleClickEvent(BaseMessage baseMessage)
    {
        String respContent = "您点击了自定义菜单!";
        TextResponse textResponse=new TextResponse(baseMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }
    public static String HandleDefaultMsgOrEvent(BaseMessage baseMessage)
    {
        String respContent = "请求不可识别，请稍候尝试！";
        TextResponse textResponse=new TextResponse(baseMessage, respContent);
        String respMsg=MessageUtil.responseToXml(textResponse);
        logger.info("***step2 Response=" + respMsg);
        return respMsg;
    }

/*
读取一个网址的所有返回内容
 */
    public static String getHttpResponse(String connectUrl) {
        BufferedReader in = null;
        StringBuffer result = null;
        try {
            URI uri = new URI(connectUrl);
            URL url = uri.toURL();
            URLConnection connection = url.openConnection();
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            connection.setRequestProperty("Charset", "utf-8");
            connection.connect();

            result = new StringBuffer();
            //读取URL的响应
            in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result.append(line);
            }

            return result.toString();

        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }
        return null;
    }

    /*
    从JSON字符串中读取某些关键信息
    JSON文件来自:http://www.imageplusplus.com/doc
     */
    public static String ReadJSONString(String jSonString){
        try {
            int imageConfidenceIndex=jSonString.indexOf("confidence",jSonString.indexOf("image"));
            int imageConfidenceIndexEnd=jSonString.indexOf(",",imageConfidenceIndex);
            String imageConfidence=jSonString.substring(imageConfidenceIndex+12,imageConfidenceIndexEnd);
            int imageConfidencePercent=(int)(Float.parseFloat(imageConfidence)*100);
            int imageValueIndex=jSonString.indexOf("value",jSonString.indexOf("image"));
            int imageValueIndexEnd=jSonString.indexOf("}",imageValueIndex)-1;
            String imageValue=jSonString.substring(imageValueIndex+8,imageValueIndexEnd).trim();

            int sceneConfidenceIndex=jSonString.indexOf("confidence",jSonString.indexOf("scene"));
            int sceneConfidenceIndexEnd=jSonString.indexOf(",",sceneConfidenceIndex);
            String sceneConfidence=jSonString.substring(sceneConfidenceIndex+12,sceneConfidenceIndexEnd);
            int sceneConfidencePercent=(int)(Float.parseFloat(sceneConfidence)*100);
            int sceneValueIndex=jSonString.indexOf("value",jSonString.indexOf("scene"));
            int sceneValueIndexEnd=jSonString.indexOf("}",sceneValueIndex)-1;
            String sceneValue=jSonString.substring(sceneValueIndex+8,sceneValueIndexEnd).trim();

            String result="这张照片有【"+imageConfidencePercent+"%】的可能是【"+imageValue+"】," +
                    "有【"+sceneConfidencePercent+"%】的可能是在【"+sceneValue+"】拍摄的。";
            return result;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "服务器秀逗了,请重试!";
    }
}
